package com.xxh.cloud.store.manager.controller;

import com.aliyun.oss.common.utils.BinaryUtil;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.param.bos.OssAliConfigBO;
import com.xxh.cloud.framework.common.param.constant.ParamValTypeEnum;
import com.xxh.cloud.store.framework.modules.base.entity.vo.OssModel;
import com.xxh.cloud.store.framework.modules.utils.AliYunOssCallbackModel;
import com.xxh.cloud.store.framework.modules.utils.AliYunOssUtils;
import com.xxh.cloud.store.framework.modules.utils.StoreHttpContextUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;

/**
 * [Web端PostObject直传]服务端签名直传并设置上传回调
 * https://help.aliyun.com/document_detail/31927.html?spm=a2c4g.11186623.6.1718.5801366ciRnatu
 * */

@Log4j2
@RestController
@RequestMapping("/store/oss/ali/web/back")
@Api(tags="APP_OSS_阿里云OSS服务端签名直传并回调服务")
public class AliYunOssWebCallBackController {

    @Autowired
    private ParamsClientService paramsClientService;

    // 放在配置文件或者其他地方
    private final String aliOss = "ALI_OSS_XX";
    private final String tencentCos = "TENCENT_COS_XX";
    private final String fileLocal = "FILE_LOCAL_XX";

    @PostMapping(value = "/callback",produces ="application/json")
    @ApiOperation("回调接口")
    public Result<OssModel> callback(@RequestBody String body) {
        if(StringUtils.isEmpty(body)) {
            return new Result<OssModel>().error();
        }
        log.debug("+++++JS上传OSS回调结果:{}",body);
        //验证签名
        HttpServletRequest request= StoreHttpContextUtils.getRequest();
        AliYunOssCallbackModel callbackModel= AliYunOssUtils.getCallbackModel(body);

        OssAliConfigBO ossAliConfig =
                paramsClientService.getParamConfig(aliOss, ParamValTypeEnum.Ali_Oss.getCode(), new OssAliConfigBO());
        try {
            boolean ret=this.verifyOSSCallbackRequest(request, body, ossAliConfig);
            if(ret==false) {
                log.debug("+++++JS上传OSS回调验签失败");
            }
        } catch (NumberFormatException | IOException e) {
            log.debug("+++++JSJS上传OSS回调验签报错{}",e.getMessage());
            return new Result<OssModel>().error();
        }
//        OssStoreInfo entity=ossStoreService.add(callbackModel, ossStoreConfig); 数据库操作
        OssModel data = new OssModel();
        data.setFileName(callbackModel.getFileName());
        data.setFileUrl(ossAliConfig.getFileUrl(callbackModel.getFileName()));
        log.debug("+++++JS上传OSS回调结果返回:{}", data);
        return new Result<OssModel>().ok(data);
    }


    /**
     * 验证上传回调的Request
     * @param request
     * @param ossCallbackBody
     * @return
     * @throws NumberFormatException
     * @throws IOException
     */
    protected boolean verifyOSSCallbackRequest(HttpServletRequest request, String ossCallbackBody,  OssAliConfigBO ossAliConfig)
            throws NumberFormatException, IOException {
        boolean ret = false;
        String autorizationInput =request.getHeader("Authorization");
        String pubKeyInput = request.getHeader("x-oss-pub-key-url");
        byte[] authorization = BinaryUtil.fromBase64String(autorizationInput);
        byte[] pubKey = BinaryUtil.fromBase64String(pubKeyInput);
        String pubKeyAddr = new String(pubKey);
        if (!pubKeyAddr.startsWith("http://gosspublic.alicdn.com/")
                && !pubKeyAddr.startsWith("https://gosspublic.alicdn.com/")) {
            System.out.println("pub key addr must be oss addrss");
            return false;
        }
        String retString = executeGet(pubKeyAddr);
        retString = retString.replace("-----BEGIN PUBLIC KEY-----", "");
        retString = retString.replace("-----END PUBLIC KEY-----", "");
        String queryString = request.getQueryString();
        String uri = request.getRequestURI();
        String path = ossAliConfig.getUriPrefix() + uri;
        String decodeUri = java.net.URLDecoder.decode(path, "UTF-8");
        String authStr = decodeUri;
        if (queryString != null && !queryString.equals("")) {
            authStr += "?" + queryString;
        }
        authStr += "\n" + ossCallbackBody;
        ret = doCheck(authStr, authorization, retString);
        return ret;
    }

    /**
     * 获取public key
     * @param url
     * @return
     */
    @SuppressWarnings({ "finally" })
    private String executeGet(String url) {
        BufferedReader in = null;
        String content = null;
        try {
            // 定义HttpClient
            @SuppressWarnings("resource")
            DefaultHttpClient client = new DefaultHttpClient();
            // 实例化HTTP方法
            HttpGet request = new HttpGet();
            request.setURI(new URI(url));
            HttpResponse response = client.execute(request);

            in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
            StringBuffer sb = new StringBuffer("");
            String line = "";
            String NL = System.getProperty("line.separator");
            while ((line = in.readLine()) != null) {
                sb.append(line + NL);
            }
            in.close();
            content = sb.toString();
        } catch (Exception e) {
        } finally {
            if (in != null) {
                try {
                    in.close();// 最后要关闭BufferedReader
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            return content;
        }
    }

    /**
     * 验证RSA
     * @param content
     * @param sign
     * @param publicKey
     * @return
     */
    public static boolean doCheck(String content, byte[] sign, String publicKey) {
        try {
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            byte[] encodedKey = BinaryUtil.fromBase64String(publicKey);
            PublicKey pubKey = keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey));
            java.security.Signature signature = java.security.Signature.getInstance("MD5withRSA");
            signature.initVerify(pubKey);
            signature.update(content.getBytes());
            boolean bverify = signature.verify(sign);
            return bverify;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return false;
    }

}
